import pandas as pd
import collections

def calc_ngrams(text: str, n: int):
    return zip(*[text[i:] for i in range(n)])

def get_ngrams(s: pd.Series, n: int, delimiter: str = ' ', merge: bool = False):
    temp = calc_ngrams(s.str.cat(sep=delimiter).split(delimiter), n)
    ngrams = collections.Counter(temp).most_common()
    if merge:
        ngrams = [(delimiter.join(i[0]),i[1]) for i in ngrams]
    return ngrams



import argparse
import logging
import csv
import sys
import os
import random
import numpy as np
import torch
from tabulate import tabulate
from torch.utils.data import DataLoader, SequentialSampler, RandomSampler, Dataset
from tqdm.auto import tqdm, trange
from transformers import (
    BertForSequenceClassification, BertTokenizer, XLMForSequenceClassification, XLMTokenizer,
    XLMRobertaForSequenceClassification, XLMRobertaTokenizer, AdamW, get_linear_schedule_with_warmup
)
from sklearn.metrics import f1_score, precision_score, recall_score
from pathlib import Path

logger = logging.getLogger(__name__)

class CustomDataset(Dataset):
    def __init__(self, input_ids, labels, present=None):
        self.input_ids = input_ids
        self.labels = labels
        self.present = present

    def __len__(self):
        return len(self.labels)

    def __getitem__(self, i):
        if self.present:
            return torch.tensor(self.input_ids[i], dtype=torch.long), torch.tensor(self.labels[i], dtype=torch.long), self.present[i]
        else:
            return torch.tensor(self.input_ids[i], dtype=torch.long), torch.tensor(self.labels[i], dtype=torch.long)

class Learner():
        
    def __init__(self, data_dir, output_dir, lr = 5e-5, train_bs = 64, eval_bs = 64, model_type = 'bert', 
                model_name = 'bert-base-multilingual-cased', save_steps = 1, max_seq_length = 256, seed = 42,
                weight_decay = 0.0, adam_epsilon = 1e-8, max_grad_norm = 1.0, num_train_epochs = 5, device = None):
        self.data_dir = Path(data_dir)
        self.output_dir = Path(output_dir)
        self.test_file = self.data_dir / 'test.txt'
        self.learning_rate = lr
        self.weight_decay = weight_decay
        self.adam_epsilon = adam_epsilon
        self.max_grad_norm = max_grad_norm
        self.num_train_epochs = num_train_epochs
        self.train_bs = train_bs
        self.eval_bs = eval_bs
        self.seed = seed
        self.model_type = model_type
        self.model_name = model_name
        self.save_steps = save_steps
        self.max_seq_length = max_seq_length
        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu') if device is None else device
        self.setup_model()

    def setup_model(self):
        # Set up logging
        logging.basicConfig(format='%(asctime)s - %(levelname)s - %(name)s -   %(message)s',
                            datefmt='%m/%d/%Y %H:%M:%S',
                            level=logging.INFO)

        # Set seed
        self.set_seed()

        # Prepare data
        self.labels = self.get_labels()
        num_labels = len(self.labels)

        # Initialize model
        tokenizer_class = {'xlm': XLMTokenizer, 'bert': BertTokenizer, 'xlm-roberta': XLMRobertaTokenizer}
        if self.model_type not in tokenizer_class.keys():
            print('Model type has to be xlm/xlm-roberta/bert')
            exit(0)
        self.tokenizer = tokenizer_class[self.model_type].from_pretrained(
            self.model_name, do_lower_case=True)
        model_class = {'xlm': XLMForSequenceClassification, 'bert': BertForSequenceClassification, 'xlm-roberta': XLMRobertaForSequenceClassification}
        self.model = model_class[self.model_type].from_pretrained(
            self.model_name, num_labels=num_labels)

        self.model.to(self.device)

    def set_seed(self):
        random.seed(self.seed)
        np.random.seed(self.seed)
        torch.manual_seed(self.seed)

    def simple_accuracy(self, preds, labels):
        return (preds == labels).mean()

    def acc_and_f1(self, preds, labels):
        acc = self.simple_accuracy(preds, labels)
        f1 = f1_score(y_true=labels, y_pred=preds, average='weighted')
        precision = precision_score(
            y_true=labels, y_pred=preds, average='weighted')
        recall = recall_score(y_true=labels, y_pred=preds, average='weighted')
        return{
            'acc': acc,
            'f1': f1,
            'acc_and_f1': (acc + f1) / 2,
            'precision': precision,
            'recall': recall
        }

    def read_examples_from_file(self, mode = 'train'):
        file_path = self.data_dir / f'{mode}.txt'
        examples = []
        with open(file_path, 'r') as infile:
            lines = infile.read().strip().split('\n')
        for line in lines:
            x = line.split('\t')
            text = x[0]
            label = x[1]
            examples.append({'text': text, 'label': label})
        if mode == 'test':
            for i in range(len(examples)):
                if examples[i]['text'] == 'not found':
                    examples[i]['present'] = False
                else:
                    examples[i]['present'] = True
        return examples

    def convert_examples_to_features(self, examples,
                                    label_list,
                                    tokenizer):

        label_map = {label: i for i, label in enumerate(label_list)}

        features = []

        for (ex_index, example) in enumerate(examples):

            sentence = example['text']
            label = example['label']

            sentence_tokens = tokenizer.tokenize(sentence)[:self.max_seq_length - 2]
            sentence_tokens = [tokenizer.cls_token] + \
                sentence_tokens + [tokenizer.sep_token]
            input_ids = tokenizer.convert_tokens_to_ids(sentence_tokens)

            label = label_map[label]
            features.append({'input_ids': input_ids,
                            'label': label})
            if 'present' in example:
                features[-1]['present'] = example['present']

        return features

    def get_labels(self):
        all_path = self.data_dir / 'train.txt'
        labels = []
        with open(all_path, 'r') as infile:
            lines = infile.read().strip().split('\n')

        for line in lines:
            splits = line.split('\t')
            label = splits[-1]
            if label not in labels:
                labels.append(label)
        return labels

    def load_and_cache_examples(self, tokenizer, labels, mode):

        logger.info(f'Creating features from dataset file at {self.data_dir}')
        examples = self.read_examples_from_file(mode)
        features = self.convert_examples_to_features(examples, labels, tokenizer)

        # Convert to Tensors and build dataset
        all_input_ids = [f['input_ids'] for f in features]
        all_labels = [f['label'] for f in features]
        args = [all_input_ids, all_labels]
        if 'present' in features[0]:
            present = [1 if f['present'] else 0 for f in features]
            args.append(present)

        dataset = CustomDataset(*args)
        return dataset

    def collate(self, examples):
        padding_value = 0

        first_sentence = [t[0] for t in examples]
        first_sentence_padded = torch.nn.utils.rnn.pad_sequence(
            first_sentence, batch_first=True, padding_value=padding_value)

        max_length = first_sentence_padded.shape[1]
        first_sentence_attn_masks = torch.stack([torch.cat([torch.ones(len(t[0]), dtype=torch.long), torch.zeros(
            max_length - len(t[0]), dtype=torch.long)]) for t in examples])

        labels = torch.stack([t[1] for t in examples])

        return first_sentence_padded, first_sentence_attn_masks, labels

    def train(self, train_dataset, valid_dataset):
        model, tokenizer, labels = self.model, self.tokenizer, self.labels
        train_sampler = RandomSampler(train_dataset)
        train_dataloader = DataLoader(
            train_dataset, sampler=train_sampler, batch_size=self.train_bs, collate_fn=self.collate)
        train_bs = self.train_bs

        # Prepare optimizer
        t_total = len(train_dataloader) * self.num_train_epochs
        no_decay = ['bias', 'LayerNorm.weight']
        optimizer_grouped_parameters = [
            {'params': [p for n, p in model.named_parameters() if not any(
                nd in n for nd in no_decay)], 'weight_decay': self.weight_decay},
            {'params': [p for n, p in model.named_parameters() if any(
                nd in n for nd in no_decay)], 'weight_decay': 0.0}
        ]
        optimizer = AdamW(optimizer_grouped_parameters,
                        lr=self.learning_rate, eps=self.adam_epsilon)
        scheduler = get_linear_schedule_with_warmup(optimizer, num_warmup_steps=t_total // 10, num_training_steps=t_total)

        # Train!
        logger.info('***** Running training *****')
        logger.info('  Num examples = %d', len(train_dataset))
        logger.info('  Num Epochs = %d', self.num_train_epochs)
        logger.info('  Instantaneous batch size per GPU = %d', train_bs)

        global_step = 0
        tr_loss, logging_loss = 0.0, 0.0
        model.zero_grad()
        train_iterator = trange(int(self.num_train_epochs), desc='Epoch')
        self.set_seed()
        best_f1_score = 0
        if not os.path.exists(self.output_dir):
            os.makedirs(self.output_dir)
        for _ in train_iterator:
            epoch_iterator = tqdm(train_dataloader, desc='Iteration')
            for step, batch in enumerate(epoch_iterator):
                model.train()
                batch = tuple(t.to(self.device) for t in batch)
                inputs = {'input_ids': batch[0],
                        'attention_mask': batch[1],
                        'labels': batch[2]}
                outputs = model(**inputs)
                # model outputs are always tuple in transformers (see doc)
                loss = outputs[0]

                loss.backward()
                torch.nn.utils.clip_grad_norm_(
                    model.parameters(), self.max_grad_norm)

                tr_loss += loss.item()
                optimizer.step()
                scheduler.step()
                model.zero_grad()
                global_step += 1

            # Checking for validation accuracy and stopping after drop in accuracy for 3 epochs
            results = self.evaluate('validation')
            if results.get('f1') > best_f1_score and self.save_steps > 0:
                best_f1_score = results.get('f1')
                model_to_save = model.module if hasattr(model, 'module') else model
                model_to_save.save_pretrained(self.output_dir)
                tokenizer.save_pretrained(self.output_dir)
                torch.save(self, os.path.join(
                    self.output_dir, 'training_args.bin'))

        return global_step, tr_loss / global_step

    def evaluate(self, mode, prefix=''):
        eval_dataset = self.load_and_cache_examples(self.tokenizer, self.labels, mode=mode)
        eval_sampler = SequentialSampler(eval_dataset)
        eval_dataloader = DataLoader(
            eval_dataset, sampler=eval_sampler, batch_size=self.eval_bs, collate_fn=self.collate)
        results = {}

        # Evaluation
        logger.info('***** Running evaluation %s *****', prefix)
        logger.info('  Num examples = %d', len(eval_dataset))
        logger.info('  Batch size = %d', self.eval_bs)
        eval_loss = 0.0
        nb_eval_steps = 0
        preds = None
        out_label_ids = None
        self.model.eval()

        for batch in tqdm(eval_dataloader, desc='Evaluating'):
            self.model.eval()
            batch = tuple(t.to(self.device) for t in batch)

            with torch.no_grad():
                inputs = {'input_ids': batch[0],
                        'attention_mask': batch[1],
                        'labels': batch[2]}
                '''print(inputs['input_ids'])
                print(inputs['attention_mask'])
                print(inputs['token_type_ids'])'''
                outputs = self.model(**inputs)
                tmp_eval_loss, logits = outputs[:2]
                eval_loss += tmp_eval_loss.mean().item()

            nb_eval_steps += 1
            # print(preds)
            if preds is None:
                preds = logits.detach().cpu().numpy()
                out_label_ids = inputs['labels'].detach().cpu().numpy()
            else:
                preds = np.append(preds, logits.detach().cpu().numpy(), axis=0)
                out_label_ids = np.append(
                    out_label_ids, inputs['labels'].detach().cpu().numpy(), axis=0)

        eval_loss = eval_loss / nb_eval_steps

        preds = np.argmax(preds, axis=1)
        # print(preds)
        if mode == 'test':
            preds_list = []
            label_map = {i: label for i, label in enumerate(self.labels)}

            for i in range(out_label_ids.shape[0]):
                # print(eval_dataset[i])
                if eval_dataset[i][2] == 0:
                    preds_list.append('not found')
                else:
                    preds_list.append(label_map[preds[i]])

            return preds_list

        else:
            result = self.acc_and_f1(preds, out_label_ids)
            results.update(result)

            logger.info('***** Eval results %s *****', prefix)
            for key in sorted(result.keys()):
                logger.info('  %s = %s', key, str(result[key]))

            return results

    def fit(self):
        # Training

        # logger.info('Training/evaluation parameters %s', args)
        train_dataset = self.load_and_cache_examples(
            self.tokenizer, self.labels, mode='train')
        valid_dataset = self.load_and_cache_examples(
            self.tokenizer, self.labels, mode='validation')
        global_step, tr_loss = self.train(train_dataset, valid_dataset)
        logger.info(' global_step = %s, average loss = %s', global_step, tr_loss)

        # Evaluation
        results = {}

        results = self.evaluate(mode='validation')
        print(tabulate(results.items(),headers=['metric','score']))
        preds = self.evaluate(mode='test')

        # Saving predictions
        output_test_predictions_file = self.output_dir / 'test_predictions.txt'
        with open(output_test_predictions_file, "w") as writer:
            writer.write('\n'.join(preds))



import string
import pandas as pd
import collections

def remove_lessthan(s: pd.Series, length: int) -> pd.Series:
    return s.apply(lambda i: ' '.join(filter(lambda j: len(j) > 3, i.split())))

def remove_punctuation(s: pd.Series, punctuation: str = string.punctuation) -> pd.Series:
    return s.str.replace(rf"([{punctuation}])+", " ")

def remove_stopwords(s: pd.Series, stopwords: list):
    return s.apply(lambda x: ' '.join([item for item in x.split() if item not in stopwords]))



import nltk

try:
    # If not present, download NLTK stopwords.
    nltk.data.find("corpora/stopwords")
except LookupError:
    nltk.download("stopwords")

from nltk.corpus import stopwords

en_stopwords = set(stopwords.words("english"))

hi_stopwords = {"अंदर","अत","अदि","अप","अपना","अपनि","अपनी","अपने","अभि","अभी","आदि","आप","इंहिं","इंहें","इंहों","इतयादि","इत्यादि","इन","इनका","इन्हीं","इन्हें","इन्हों","इस","इसका","इसकि","इसकी","इसके","इसमें","इसि","इसी","इसे","उंहिं","उंहें","उंहों","उन","उनका","उनकि","उनकी","उनके","उनको","उन्हीं","उन्हें","उन्हों","उस","उसके","उसि","उसी","उसे","एक","एवं","एस","एसे","ऐसे","ओर","और","कइ","कई","कर","करता","करते","करना","करने","करें","कहते","कहा","का","काफि","काफ़ी","कि","किंहें","किंहों","कितना","किन्हें","किन्हों","किया","किर","किस","किसि","किसी","किसे","की","कुछ","कुल","के","को","कोइ","कोई","कोन","कोनसा","कौन","कौनसा","गया","घर","जब","जहाँ","जहां","जा","जिंहें","जिंहों","जितना","जिधर","जिन","जिन्हें","जिन्हों","जिस","जिसे","जीधर","जेसा","जेसे","जैसा","जैसे","जो","तक","तब","तरह","तिंहें","तिंहों","तिन","तिन्हें","तिन्हों","तिस","तिसे","तो","था","थि","थी","थे","दबारा","दवारा","दिया","दुसरा","दुसरे","दूसरे","दो","द्वारा","न","नहिं","नहीं","ना","निचे","निहायत","नीचे","ने","पर","पहले","पुरा","पूरा","पे","फिर","बनि","बनी","बहि","बही","बहुत","बाद","बाला","बिलकुल","भि","भितर","भी","भीतर","मगर","मानो","मे","में","यदि","यह","यहाँ","यहां","यहि","यही","या","यिह","ये","रखें","रवासा","रहा","रहे","ऱ्वासा","लिए","लिये","लेकिन","व","वगेरह","वरग","वर्ग","वह","वहाँ","वहां","वहिं","वहीं","वाले","वुह","वे","वग़ैरह","संग","सकता","सकते","सबसे","सभि","सभी","साथ","साबुत","साभ","सारा","से","सो","हि","ही","हुअ","हुआ","हुइ","हुई","हुए","हे","हें","है","हैं","हो","होता","होति","होती","होते","होना","होने"}

hinglish_stopwords = {'a', 'aadi', 'aaj', 'aap', 'aapne', 'aata', 'aati', 'aaya', 'aaye', 'ab', 'abbe', 'abbey', 'abe', 'abhi', 'able', 'about', 'above', 'accha', 'according', 'accordingly', 'acha', 'achcha', 'across', 'actually', 'after', 'afterwards', 'again', 'against', 'agar', 'ain', 'aint', "ain't", 'aisa', 'aise', 'aisi', 'alag', 'all', 'allow', 'allows', 'almost', 'alone', 'along', 'already', 'also', 'although', 'always', 'am', 'among', 'amongst', 'an', 'and', 'andar', 'another', 'any', 'anybody', 'anyhow', 'anyone', 'anything', 'anyway', 'anyways', 'anywhere', 'ap', 'apan', 'apart', 'apna', 'apnaa', 'apne', 'apni', 'appear', 'are', 'aren', 'arent', "aren't", 'around', 'arre', 'as', 'aside', 'ask', 'asking', 'at', 'aur', 'avum', 'aya', 'aye', 'baad', 'baar', 'bad', 'bahut', 'bana', 'banae', 'banai', 'banao', 'banaya', 'banaye', 'banayi', 'banda', 'bande', 'bandi', 'bane', 'bani', 'bas', 'bata', 'batao', 'bc', 'be', 'became', 'because', 'become', 'becomes', 'becoming', 'been', 'before', 'beforehand', 'behind', 'being', 'below', 'beside', 'besides', 'best', 'better', 'between', 'beyond', 'bhai', 'bheetar', 'bhi', 'bhitar', 'bht', 'bilkul', 'bohot', 'bol', 'bola', 'bole', 'boli', 'bolo', 'bolta', 'bolte', 'bolti', 'both', 'brief', 'bro', 'btw', 'but', 'by', 'came', 'can', 'cannot', 'cant', "can't", 'cause', 'causes', 'certain', 'certainly', 'chahiye', 'chaiye', 'chal', 'chalega', 'chhaiye', 'clearly', "c'mon", 'com', 'come', 'comes', 'could', 'couldn', 'couldnt', "couldn't", 'd', 'de', 'dede', 'dega', 'degi', 'dekh', 'dekha', 'dekhe', 'dekhi', 'dekho', 'denge', 'dhang', 'di', 'did', 'didn', 'didnt', "didn't", 'dijiye', 'diya', 'diyaa', 'diye', 'diyo', 'do', 'does', 'doesn', 'doesnt', "doesn't", 'doing', 'done', 'dono', 'dont', "don't", 'doosra', 'doosre', 'down', 'downwards', 'dude', 'dunga', 'dungi', 'during', 'dusra', 'dusre', 'dusri', 'dvaara', 'dvara', 'dwaara', 'dwara', 'each', 'edu', 'eg', 'eight', 'either', 'ek', 'else', 'elsewhere', 'enough', 'etc', 'even', 'ever', 'every', 'everybody', 'everyone', 'everything', 'everywhere', 'ex', 'exactly', 'example', 'except', 'far', 'few', 'fifth', 'fir', 'first', 'five', 'followed', 'following', 'follows', 'for', 'forth', 'four', 'from', 'further', 'furthermore', 'gaya', 'gaye', 'gayi', 'get', 'gets', 'getting', 'ghar', 'given', 'gives', 'go', 'goes', 'going', 'gone', 'good', 'got', 'gotten', 'greetings', 'haan', 'had', 'hadd', 'hadn', 'hadnt', "hadn't", 'hai', 'hain', 'hamara', 'hamare', 'hamari', 'hamne', 'han', 'happens', 'har', 'hardly', 'has', 'hasn', 'hasnt', "hasn't", 'have', 'haven', 'havent', "haven't", 'having', 'he', 'hello', 'help', 'hence', 'her', 'here', 'hereafter', 'hereby', 'herein', "here's", 'hereupon', 'hers', 'herself', "he's", 'hi', 'him', 'himself', 'his', 'hither', 'hm', 'hmm', 'ho', 'hoga', 'hoge', 'hogi', 'hona', 'honaa', 'hone', 'honge', 'hongi', 'honi', 'hopefully', 'hota', 'hotaa', 'hote', 'hoti', 'how', 'howbeit', 'however', 'hoyenge', 'hoyengi', 'hu', 'hua', 'hue', 'huh', 'hui', 'hum', 'humein', 'humne', 'hun', 'huye', 'huyi', 'i', "i'd", 'idk', 'ie', 'if', "i'll", "i'm", 'imo', 'in', 'inasmuch', 'inc', 'inhe', 'inhi', 'inho', 'inka', 'inkaa', 'inke', 'inki', 'inn', 'inner', 'inse', 'insofar', 'into', 'inward', 'is', 'ise', 'isi', 'iska', 'iskaa', 'iske', 'iski', 'isme', 'isn', 'isne', 'isnt', "isn't", 'iss', 'isse', 'issi', 'isski', 'it', "it'd", "it'll", 'itna', 'itne', 'itni', 'itno', 'its', "it's", 'itself', 'ityaadi', 'ityadi', "i've", 'ja', 'jaa', 'jab', 'jabh', 'jaha', 'jahaan', 'jahan', 'jaisa', 'jaise', 'jaisi', 'jata', 'jayega', 'jidhar', 'jin', 'jinhe', 'jinhi', 'jinho', 'jinhone', 'jinka', 'jinke', 'jinki', 'jinn', 'jis', 'jise', 'jiska', 'jiske', 'jiski', 'jisme', 'jiss', 'jisse', 'jitna', 'jitne', 'jitni', 'jo', 'just', 'jyaada', 'jyada', 'k', 'ka', 'kaafi', 'kab', 'kabhi', 'kafi', 'kaha', 'kahaa', 'kahaan', 'kahan', 'kahi', 'kahin', 'kahte', 'kaisa', 'kaise', 'kaisi', 'kal', 'kam', 'kar', 'kara', 'kare', 'karega', 'karegi', 'karen', 'karenge', 'kari', 'karke', 'karna', 'karne', 'karni', 'karo', 'karta', 'karte', 'karti', 'karu', 'karun', 'karunga', 'karungi', 'kaun', 'kaunsa', 'kayi', 'kch', 'ke', 'keep', 'keeps', 'keh', 'kehte', 'kept', 'khud', 'ki', 'kin', 'kine', 'kinhe', 'kinho', 'kinka', 'kinke', 'kinki', 'kinko', 'kinn', 'kino', 'kis', 'kise', 'kisi', 'kiska', 'kiske', 'kiski', 'kisko', 'kisliye', 'kisne', 'kitna', 'kitne', 'kitni', 'kitno', 'kiya', 'kiye', 'know', 'known', 'knows', 'ko', 'koi', 'kon', 'konsa', 'koyi', 'krna', 'krne', 'kuch', 'kuchch', 'kuchh', 'kul', 'kull', 'kya', 'kyaa', 'kyu', 'kyuki', 'kyun', 'kyunki', 'lagta', 'lagte', 'lagti', 'last', 'lately', 'later', 'le', 'least', 'lekar', 'lekin', 'less', 'lest', 'let', "let's", 'li', 'like', 'liked', 'likely', 'little', 'liya', 'liye', 'll', 'lo', 'log', 'logon', 'lol', 'look', 'looking', 'looks', 'ltd', 'lunga', 'm', 'maan', 'maana', 'maane', 'maani', 'maano', 'magar', 'mai', 'main', 'maine', 'mainly', 'mana', 'mane', 'mani', 'mano', 'many', 'mat', 'may', 'maybe', 'me', 'mean', 'meanwhile', 'mein', 'mera', 'mere', 'merely', 'meri', 'might', 'mightn', 'mightnt', "mightn't", 'mil', 'mjhe', 'more', 'moreover', 'most', 'mostly', 'much', 'mujhe', 'must', 'mustn', 'mustnt', "mustn't", 'my', 'myself', 'na', 'naa', 'naah', 'nahi', 'nahin', 'nai', 'name', 'namely', 'nd', 'ne', 'near', 'nearly', 'necessary', 'neeche', 'need', 'needn', 'neednt', "needn't", 'needs', 'neither', 'never', 'nevertheless', 'new', 'next', 'nhi', 'nine', 'no', 'nobody', 'non', 'none', 'noone', 'nope', 'nor', 'normally', 'not', 'nothing', 'novel', 'now', 'nowhere', 'o', 'obviously', 'of', 'off', 'often', 'oh', 'ok', 'okay', 'old', 'on', 'once', 'one', 'ones', 'only', 'onto', 'or', 'other', 'others', 'otherwise', 'ought', 'our', 'ours', 'ourselves', 'out', 'outside', 'over', 'overall', 'own', 'par', 'pata', 'pe', 'pehla', 'pehle', 'pehli', 'people', 'per', 'perhaps', 'phla', 'phle', 'phli', 'placed', 'please', 'plus', 'poora', 'poori', 'provides', 'pura', 'puri', 'q', 'que', 'quite', 'raha', 'rahaa', 'rahe', 'rahi', 'rakh', 'rakha', 'rakhe', 'rakhen', 'rakhi', 'rakho', 'rather', 're', 'really', 'reasonably', 'regarding', 'regardless', 'regards', 'rehte', 'rha', 'rhaa', 'rhe', 'rhi', 'ri', 'right', 's', 'sa', 'saara', 'saare', 'saath', 'sab', 'sabhi', 'sabse', 'sahi', 'said', 'sakta', 'saktaa', 'sakte', 'sakti', 'same', 'sang', 'sara', 'sath', 'saw', 'say', 'saying', 'says', 'se', 'second', 'secondly', 'see', 'seeing', 'seem', 'seemed', 'seeming', 'seems', 'seen', 'self', 'selves', 'sensible', 'sent', 'serious', 'seriously', 'seven', 'several', 'shall', 'shan', 'shant', "shan't", 'she', "she's", 'should', 'shouldn', 'shouldnt', "shouldn't", "should've", 'si', 'since', 'six', 'so', 'soch', 'some', 'somebody', 'somehow', 'someone', 'something', 'sometime', 'sometimes', 'somewhat', 'somewhere', 'soon', 'still', 'sub', 'such', 'sup', 'sure', 't', 'tab', 'tabh', 'tak', 'take', 'taken', 'tarah', 'teen', 'teeno', 'teesra', 'teesre', 'teesri', 'tell', 'tends', 'tera', 'tere', 'teri', 'th', 'tha', 'than', 'thank', 'thanks', 'thanx', 'that', "that'll", 'thats', "that's", 'the', 'theek', 'their', 'theirs', 'them', 'themselves', 'then', 'thence', 'there', 'thereafter', 'thereby', 'therefore', 'therein', 'theres', "there's", 'thereupon', 'these', 'they', "they'd", "they'll", "they're", "they've", 'thi', 'thik', 'thing', 'think', 'thinking', 'third', 'this', 'tho', 'thoda', 'thodi', 'thorough', 'thoroughly', 'those', 'though', 'thought', 'three', 'through', 'throughout', 'thru', 'thus', 'tjhe', 'to', 'together', 'toh', 'too', 'took', 'toward', 'towards', 'tried', 'tries', 'true', 'truly', 'try', 'trying', 'tu', 'tujhe', 'tum', 'tumhara', 'tumhare', 'tumhari', 'tune', 'twice', 'two', 'um', 'umm', 'un', 'under', 'unhe', 'unhi', 'unho', 'unhone', 'unka', 'unkaa', 'unke', 'unki', 'unko', 'unless', 'unlikely', 'unn', 'unse', 'until', 'unto', 'up', 'upar', 'upon', 'us', 'use', 'used', 'useful', 'uses', 'usi', 'using', 'uska', 'uske', 'usne', 'uss', 'usse', 'ussi', 'usually', 'vaala', 'vaale', 'vaali', 'vahaan', 'vahan', 'vahi', 'vahin', 'vaisa', 'vaise', 'vaisi', 'vala', 'vale', 'vali', 'various', 've', 'very', 'via', 'viz', 'vo', 'waala', 'waale', 'waali', 'wagaira', 'wagairah', 'wagerah', 'waha', 'wahaan', 'wahan', 'wahi', 'wahin', 'waisa', 'waise', 'waisi', 'wala', 'wale', 'wali', 'want', 'wants', 'was', 'wasn', 'wasnt', "wasn't", 'way', 'we', "we'd", 'well', "we'll", 'went', 'were', "we're", 'weren', 'werent', "weren't", "we've", 'what', 'whatever', "what's", 'when', 'whence', 'whenever', 'where', 'whereafter', 'whereas', 'whereby', 'wherein', "where's", 'whereupon', 'wherever', 'whether', 'which', 'while', 'who', 'whoever', 'whole', 'whom', "who's", 'whose', 'why', 'will', 'willing', 'with', 'within', 'without', 'wo', 'woh', 'wohi', 'won', 'wont', "won't", 'would', 'wouldn', 'wouldnt', "wouldn't", 'y', 'ya', 'yadi', 'yah', 'yaha', 'yahaan', 'yahan', 'yahi', 'yahin', 'ye', 'yeah', 'yeh', 'yehi', 'yes', 'yet', 'you', "you'd", "you'll", 'your', "you're", 'yours', 'yourself', 'yourselves', "you've", 'yup'}
